/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.ui;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.ui.UiPlugin;
import benten.ui.UiPlugin.Preference;

/**
 * The activator class controls the plug-in life cycle
 */
public class CatGlossaryUiPlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = CatGlossaryUiPlugin.class.getPackage().getName();

	// The shared instance
	private static CatGlossaryUiPlugin plugin;

	private BentenGlossaryEngine glossaryEngine;

	/**
	 * The constructor
	 */
	public CatGlossaryUiPlugin() {
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		loadExtensions();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static CatGlossaryUiPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path
	 *
	 * @param path
	 *            the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}

	/**
	 * Logs the given message.
	 *
	 * @param message
	 *            the message to log
	 */
	public static void log(String message) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, message));
	}

	/**
	 * Logs the given exception.
	 *
	 * @param ex
	 *            the exception to log
	 */
	public static void log(final Throwable ex) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, -1, ex.toString(), ex));
	}

	public void loadExtensions() {
		final BentenGlossaryEngine glossaryEngine = newGlossaryEngine();
		if (glossaryEngine != null) {
			glossaryEngine.load();
			this.glossaryEngine = glossaryEngine;
		}
	}

	/**
	 * このプラグインが保持している グロッサリー・エンジンを取得します。
	 *
	 * @return グロッサリー・エンジンのインスタンス。
	 */
	public BentenGlossaryEngine getGlossaryEngine() {
		return glossaryEngine;
	}

	/**
	 * グロッサリー・エンジンのインスタンスを新規に作成します。
	 *
	 * 所定のルールに従って拡張ポイントを検索して グロッサリー・エンジンのインスタンスを作成します。
	 *
	 * @return グロッサリー・エンジンのインスタンス。
	 */
	public BentenGlossaryEngine newGlossaryEngine() {
		final Map<String, BentenGlossaryEngine> glossaryMap = new HashMap<String, BentenGlossaryEngine>();

		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.glossary.engine");
		System.out.println(point);
		if (point == null) {
			return null;
		}

		final IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			final IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				final IConfigurationElement cfgElem = cfgElems[j];
				try {
					final String name = cfgElem.getAttribute("name");
					final Object obj = cfgElem.createExecutableExtension("class");
					if (obj instanceof BentenGlossaryEngine) {
						glossaryMap.put(name, (BentenGlossaryEngine) obj);
					}
				} catch (final CoreException e) {
					log(e);
				}
			}
		}

		final IPreferenceStore store = UiPlugin.getDefault().getPreferenceStore();
		// TODO TM_NAME を引き当ててはなりません。グロッサリー名を取得する必要があります。
		final String tmName = store.getString(Preference.TM_NAME.name());
		return glossaryMap.get(tmName);
	}
}
