/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.ui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class GlossaryEntryDialog extends Dialog {

	private String src;
	private String loc;
	private String note;
	private boolean ignoreCase = true;

	private Text srcText;
	private Text locText;
	private Text noteText;
	private Button ignoreCaseCheckBox;

	public String getSrc() {
		return src;
	}

	public void setSrc(String src) {
		this.src = src;
	}

	public String getLoc() {
		return loc;
	}

	public void setLoc(String loc) {
		this.loc = loc;
	}

	public String getNote() {
		return note;
	}

	public void setNote(String note) {
		this.note = note;
	}

	public boolean isIgnoreCase() {
		return ignoreCase;
	}

	public void setIgnoreCase(boolean ignoreCase) {
		this.ignoreCase = ignoreCase;
	}

	public GlossaryEntryDialog(Shell parentShell) {
		super(parentShell);
	}

	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Add New Grossary Entry");
	}

	@Override
	protected Point getInitialSize() {
		Rectangle bounds = getShell().getBounds();
		System.out.println(bounds);
		return new Point(400, 200);
	}

	@Override
	protected Control createDialogArea(Composite parent) {

		Composite container = (Composite) super.createDialogArea(parent);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		container.setLayout(gridLayout);

		final Label descLabel = new Label(container, SWT.NONE);
		descLabel.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, false, false, 2, 1));
		descLabel.setText("Test String");

		final Label srcLabel = new Label(container, SWT.NONE);
		srcLabel.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false));
		srcLabel.setText("src:");

		srcText = new Text(container, SWT.BORDER);
		srcText.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));

		final Label locLabel = new Label(container, SWT.NONE);
		locLabel.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false));
		locLabel.setText("loc:");

		locText = new Text(container, SWT.BORDER);
		locText.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));

		final Label noteLabel = new Label(container, SWT.NONE);
		final GridData gridData = new GridData(GridData.END, GridData.CENTER, false, false);
		gridData.horizontalIndent = 10;
		noteLabel.setLayoutData(gridData);
		noteLabel.setText("note:");

		noteText = new Text(container, SWT.BORDER);
		noteText.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));

		ignoreCaseCheckBox = new Button(parent, SWT.CHECK);
		final GridData ignoreCaseGrid = new GridData(GridData.BEGINNING, GridData.CENTER, false, false, 2, 1);
		ignoreCaseGrid.horizontalIndent = 10;
		ignoreCaseCheckBox.setLayoutData(ignoreCaseGrid);
		ignoreCaseCheckBox.setText("ignore case");

		initContent();

		return container;

	}

	private void initContent() {
		srcText.setText(src != null ? src : "");
		srcText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				src = srcText.getText();
			}
		});

		locText.setText(loc != null ? loc : "");
		locText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				loc = locText.getText();
			}
		});

		noteText.setText(note != null ? note : "");
		noteText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				note = noteText.getText();
			}
		});

		ignoreCaseCheckBox.setSelection(ignoreCase);
		ignoreCaseCheckBox.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}

			public void widgetSelected(SelectionEvent e) {
				ignoreCase = ignoreCaseCheckBox.getSelection();
			}
		});

		if ("".equals(srcText.getText())) {
			srcText.setFocus();
		} else {
			locText.setFocus();
		}
	}
}
